/*
             LUFA Library
     Copyright (C) Dean Camera, 2010.
              
  dean [at] fourwalledcubicle [dot] com
      www.fourwalledcubicle.com
*/

/*
  Copyright 2010  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this 
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in 
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting 
  documentation, and that the name of the author not be used in 
  advertising or publicity pertaining to distribution of the 
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

#include <util/delay.h>

/** \file
 *
 *  Main source file for the VirtualSerial demo. This file contains the main tasks of
 *  the demo and is responsible for the initial application hardware configuration.
 */
 
#include "VirtualSerial.h"

/** LUFA CDC Class driver interface configuration and state information. This structure is
 *  passed to all CDC Class driver functions, so that multiple instances of the same class
 *  within a device can be differentiated from one another.
 */
USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface =
	{
		.Config =
			{
				.ControlInterfaceNumber         = 0,

				.DataINEndpointNumber           = CDC_TX_EPNUM,
				.DataINEndpointSize             = CDC_TXRX_EPSIZE,
				.DataINEndpointDoubleBank       = false,

				.DataOUTEndpointNumber          = CDC_RX_EPNUM,
				.DataOUTEndpointSize            = CDC_TXRX_EPSIZE,
				.DataOUTEndpointDoubleBank      = false,

				.NotificationEndpointNumber     = CDC_NOTIFICATION_EPNUM,
				.NotificationEndpointSize       = CDC_NOTIFICATION_EPSIZE,
				.NotificationEndpointDoubleBank = false,
			},
	};

/** Standard file stream for the CDC interface when set up, so that the virtual CDC COM port can be
 *  used like any regular character stream in the C APIs
 */
static FILE USBSerialStream;

/** Main program entry point. This routine contains the overall program flow, including initial
 *  setup of all components and the main program loop.
 */
int main(void)
{
  int t;
  int c;
  char inputbuffer[0x20];
  int nchars;
  unsigned char intro_sent;

  SetupHardware();
	
  /* Create a regular character stream for the interface so that it can be used with the stdio.h functions */
  CDC_Device_CreateStream(&VirtualSerial_CDC_Interface, &USBSerialStream);

  sei();

  t = 0;
  nchars = 0;
  DDRB = 1;
  intro_sent = 0;

  for (;;)
    {
      if (VirtualSerial_CDC_Interface.State.ControlLineStates.HostToDevice & 
	  CDC_CONTROL_LINE_OUT_DTR) {
	if (!intro_sent) 
	  fputs("CDC led controller\r\n", &USBSerialStream);
	intro_sent = 1;
      }else {
	intro_sent = 0;
      }


      /* Must throw away unused bytes from the host, or it will lock up while waiting for the device */
      c = CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface);
      if (c > 0) {
	if (nchars > 0x1e){
	  CDC_Device_SendByte (&VirtualSerial_CDC_Interface, '\b');
	  nchars = 0; 
	} else {
	  
	  inputbuffer[nchars++] = c;
	  if ((c == '\r') || 
	      (c == '\n')) {
	    switch (inputbuffer[0]) {
	    case 'a':
	      c = inputbuffer[1];
	      switch (c) {
	      case '1':PORTD |= 0x03;break;
	      case '2':PORTD |= 0x0c;break;
	      case '3':PORTD |= 0x30;break;
	      case '4':PORTB |= 0x01;break;
	      }
	      break;
	    case 'u':
	      c = inputbuffer[1];
	      switch (c) {
	      case '1':PORTD &= ~0x03;break;
	      case '2':PORTD &= ~0x0c;break;
	      case '3':PORTD &= ~0x30;break;
	      case '4':PORTB &= ~0x01;break;
	      }
	      break;
	    }
	    fputs ("\r\n> ", &USBSerialStream);
	    nchars = 0;
	  } else {
	    CDC_Device_SendByte (&VirtualSerial_CDC_Interface, c);
	  }
	}
      }
	  
      CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
      USB_USBTask();
      _delay_ms (1);
      t ++;
      if (t == 1000) PORTB |= 0x01;
      if (t == 2000) {
	PORTB &= ~0x01;
        t = 0;
      }	
    }
}

/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware(void)
{
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
	clock_prescale_set(clock_div_1);

	/* Hardware Initialization */
	USB_Init();
}


/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
}

/** Event handler for the library USB Unhandled Control Request event. */
void EVENT_USB_Device_UnhandledControlRequest(void)
{
	CDC_Device_ProcessControlRequest(&VirtualSerial_CDC_Interface);
}
