/*
             LUFA Library
     Copyright (C) Dean Camera, 2010.
              
  dean [at] fourwalledcubicle [dot] com
      www.fourwalledcubicle.com
*/

/*
  Copyright 2010  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this 
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in 
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting 
  documentation, and that the name of the author not be used in 
  advertising or publicity pertaining to distribution of the 
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

#include <util/delay.h>

/** \file
 *
 *  Main source file for the VirtualSerial demo. This file contains
 *  the main tasks of the demo and is responsible for the initial
 *  application hardware configuration.
 */
 
#include "VirtualSerial.h"
#include "bits.h"

/** LUFA CDC Class driver interface configuration and state
 *  information. This structure is passed to all CDC Class driver
 *  functions, so that multiple instances of the same class within a
 *  device can be differentiated from one another.
 */
USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface =
  {
    .Config =
    {
      .ControlInterfaceNumber         = 0,

      .DataINEndpointNumber           = CDC_TX_EPNUM,
      .DataINEndpointSize             = CDC_TXRX_EPSIZE,
      .DataINEndpointDoubleBank       = false,

      .DataOUTEndpointNumber          = CDC_RX_EPNUM,
      .DataOUTEndpointSize            = CDC_TXRX_EPSIZE,
      .DataOUTEndpointDoubleBank      = false,

      .NotificationEndpointNumber     = CDC_NOTIFICATION_EPNUM,
      .NotificationEndpointSize       = CDC_NOTIFICATION_EPSIZE,
      .NotificationEndpointDoubleBank = false,
    },
  };

/** Standard file stream for the CDC interface when set up, so that
 *  the virtual CDC COM port can be used like any regular character
 *  stream in the C APIs
 */
static FILE USBSerialStream;

void delay_ms(unsigned int ms)
/* delay for a minimum of <ms> */
{
  // we use a calibrated macro. This is more
  // accurate and not so much compiler dependent
  // as self made code.
  while(ms){
    _delay_ms(0.98);
    ms--;
  }
}




void hexnibble (unsigned char b)
{
  b += '0';
  if (b <= '9') {
    fputc (b, &USBSerialStream);
    return;
  }
  b += 7;
  fputc (b, &USBSerialStream);
  return;
}

void hexbyte (unsigned char v)
{
  hexnibble (v >> 4);
  hexnibble (v & 0xf);
}

void hexshort (unsigned int v)
{
  hexbyte (v >> 8);
  hexbyte (v & 0xff);
}


unsigned char lbp;
char inputbuffer[0x20];

unsigned char lb_peekchar (void)
{
  return inputbuffer[lbp];
}

unsigned char lb_getchar (void)
{
  return inputbuffer[lbp++];
}

unsigned char conv_nibble (unsigned char c)
{
  if (c >= 'A') {
    return (c & ~0x20) - 'A' + 10;
  } 
  return c - '0';
}


int lb_gethex(void)
{
  int v = 0;
  unsigned char c;

  while (lb_peekchar() == ' ') lb_getchar (); 
  
  while ( ((c = lb_getchar ()) != ' ') &&
	  (c != '\n') && 
	  (c != '\r')) {
    v = (v << 4) | conv_nibble (c);
  }
  return v; 
}


#define MULTIO
//#define HDDPOWER

#ifdef MULTIO
unsigned char outputs [] = { 
  pD1, pD2, pD3, pD4, pD5, pD6,
  pB1, pB0, pB3, pB2, pB5, pB4, pB7, pB6,
  pC6, pC7
};
#endif
#ifdef HDDPOWER
unsigned char outputs [] = { 
  pD0,  pD1, pD2, pD3, pD4, pD5, pD6
};
#endif

#define NOUTPUTS (sizeof(outputs)/sizeof(outputs[0]))



unsigned char mode[NOUTPUTS];
unsigned char aval[NOUTPUTS];
unsigned char cpos[NOUTPUTS];


#define LED1 pC2
#define LED2 pD0

void process_line (char *inputbuffer)
{
  unsigned short i;
  int len; 

  switch (lb_getchar()) {
  case 'c':
    i = lb_gethex ();
    mode[i] = 0;
    clr_bit (outputs[i]);
    break;
  case 's':
    i = lb_gethex ();
    mode[i] = 0;
    set_bit (outputs[i]);
    break;
  case 'a':
    i = lb_gethex ();
    aval[i] = lb_gethex ();
    mode[i] = 1;
    break;
  case 'p':
    i = lb_gethex ();
    len = lb_gethex ();
    set_bit (outputs[i]);
    delay_ms (len);
    clr_bit (outputs[i]);
    break;
  case 't':
    i = lb_gethex ();
    fputs ("got: ", &USBSerialStream);
    hexshort (i); 
    break;
  case 'w':
    i = lb_gethex ();
    make_output (pC6); // OC1A
    TCCR1A = (2 << 6) | (1 << 0); // normal PWM // fast PWM 8 bit. 
    TCCR1B = (1 << 3) | (1 << 0); // FAST PWM 8 bit // clock source = sysclk/1.
    OCR1A = i; 
    break;
  case 'z':
    //UsbDisable ();
    __asm ("jmp 0x3000;");
    break;
  }
}

void do_loopfunc (void)
{
  unsigned char i;

  for (i=0;i<NOUTPUTS;i++) {
    if (mode[i]) {
      cpos[i] += aval[i];
      if (cpos [i] >= 0x80) {
	cpos[i] -= 0x80;
	set_bit (outputs[i]);
      } else {
	clr_bit (outputs[i]);
      }
    }
  }
}

  
/** Main program entry point. This routine contains the overall
 *  program flow, including initial setup of all components and the
 *  main program loop.
 */
int main(void)
{
  int t;
  int c;

  int nchars;
  unsigned char intro_sent;

  SetupHardware();
	
  /* Create a regular character stream for the interface so that it can be used with the stdio.h functions */
  CDC_Device_CreateStream(&VirtualSerial_CDC_Interface, &USBSerialStream);


  t = 0;
  nchars = 0;

  //  DDRB = _BV(PB0); // led is output, sensors are input. 

  //#define SENSOR_MASK _BV((PB1) | _BV (PB2) | _BV(PB3))
#define SENSOR_MASK 0x0e

  //PORTD = 0;
  //DDRD = 0x3f;

  //PORTB = SENSOR_MASK; // enable the pullups. 

  //PCICR = _BV(PCIE0);
  //PCMSK0 = SENSOR_MASK; // Set the sensors up for a PCINT. 


  //OCR1A = 1000-1; // reload counter/interrupt at 16000 Hz. 

  //TCCR1A = 0;
  //TCCR1B = (1 << CS10) | (1 << WGM12); // Mode 4 CTC. 
  //TCCR1B = (1 << CS10); // Mode 4 CTC. 
  //TCCR1C = 0;

  //OCR1B = 20;

  //TIMSK1 = _BV (TOIE1);
  
  // TIMSK1 = _BV (OCIE1B);

  //EICRA = 0;
  //EIMSK = 0; 
  sei (); 

  make_output (LED1);
  make_output (LED2);

  intro_sent = 0;

  for (;;) {
    if (VirtualSerial_CDC_Interface.State.ControlLineStates.HostToDevice & 
	CDC_CONTROL_LINE_OUT_DTR) {
      if (!intro_sent) {
	fputs("CDC usbio", &USBSerialStream);
#ifdef HDDPOWER
	fputs(" hddpower", &USBSerialStream);
#endif
#ifdef MULTIO
	fputs(" multiio", &USBSerialStream);
#endif
	fputs("\r\n> ", &USBSerialStream);
      }
      intro_sent = 1;
    }else {
      intro_sent = 0;
    }

    c = CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface);

    if (c > 0) {
      if (nchars > 0x1e){
	CDC_Device_SendByte (&VirtualSerial_CDC_Interface, '\b');
	nchars = 0; 
      } else {
	inputbuffer[nchars++] = c;
	if ((c == '\r') || 
	    (c == '\n')) {
	  lbp = 0;
	  process_line (inputbuffer);
	  fputs ("\r\n> ", &USBSerialStream);
	  nchars = 0;
	} else {
	  CDC_Device_SendByte (&VirtualSerial_CDC_Interface, c);
	}
      }
    }
    //test3
    CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
    USB_USBTask();
    _delay_ms (0.1);
    do_loopfunc ();
    t++;
    if (t > 5000) set_bit (LED1);
    else          clr_bit (LED1);
    if (t > 10000) t = 0;

  }
}


/** Configures the board hardware and chip peripherals for the demo's functionality. */
void SetupHardware(void)
{
  unsigned char i;

  /* Disable watchdog if enabled by bootloader/fuses */
  MCUSR &= ~(1 << WDRF);
  wdt_disable();

  /* Disable clock division */
  clock_prescale_set(clock_div_1);

  /* Hardware Initialization */
  USB_Init();

  for (i=0;i<NOUTPUTS;i++)
    make_output (outputs[i]);

}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{

	if (!(CDC_Device_ConfigureEndpoints(&VirtualSerial_CDC_Interface))) {
//	  LEDs_SetAllLEDs(LEDMASK_USB_ERROR);
	/* nothing now */
	
	}
}

/** Event handler for the library USB Unhandled Control Request event. */
void EVENT_USB_Device_UnhandledControlRequest(void)
{
	CDC_Device_ProcessControlRequest(&VirtualSerial_CDC_Interface);
}
